@@ -234,6 +234,7 @@ static void test_incorrect_api_usage(void)
     result = pCryptCreateHash(hProv, CALG_SHA, 0, 0, &hHash);
     ok (result, "%d\n", GetLastError());
     if (!result) return;
+    pCryptDestroyHash(hHash);
 
     result = pCryptGenKey(hProv, CALG_RC4, 0, &hKey);
     ok (result, "%d\n", GetLastError());
@@ -557,6 +558,7 @@ static void test_enum_providers(void)
 	    ok(!strcmp(pszProvName, provider), "expected %s, got %s\n", pszProvName, provider);
 	ok(cbName==providerLen, "expected %d, got %d\n", cbName, providerLen);
 
+	LocalFree(pszProvName);
 	LocalFree(provider);
 }
 
@@ -844,6 +846,7 @@ static void test_get_default_provider(void)
 	    ok(!strcmp(pszProvName, provName), "expected %s, got %s\n", pszProvName, provName);
 	ok(provNameSize==cbProvName, "expected %d, got %d\n", cbProvName, provNameSize);
 
+	LocalFree(pszProvName);
 	LocalFree(provName);
 }
 
@@ -956,70 +959,78 @@ static void test_rc2_keylen(void)
     ret = pCryptAcquireContextA(&provider, NULL, NULL,
                                 PROV_RSA_FULL, CRYPT_VERIFYCONTEXT);
     ok(ret, "CryptAcquireContext error %u\n", GetLastError());
+    if (ret)
+    {
+        key_blob.header.bType = PLAINTEXTKEYBLOB;
+        key_blob.header.bVersion = CUR_BLOB_VERSION;
+        key_blob.header.reserved = 0;
+        key_blob.header.aiKeyAlg = CALG_RC2;
+        key_blob.key_size = sizeof(key);
+        memcpy(key_blob.key_data, key, key_length);
 
-    key_blob.header.bType = PLAINTEXTKEYBLOB;
-    key_blob.header.bVersion = CUR_BLOB_VERSION;
-    key_blob.header.reserved = 0;
-    key_blob.header.aiKeyAlg = CALG_RC2;
-    key_blob.key_size = sizeof(key);
-    memcpy(key_blob.key_data, key, key_length);
+        /* Importing a 16-byte key works with the default provider. */
+        SetLastError(0xdeadbeef);
+        ret = pCryptImportKey(provider, (BYTE*)&key_blob,
+                          sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
+                          0, CRYPT_IPSEC_HMAC_KEY, &hkey);
+        /* CRYPT_IPSEC_HMAC_KEY is not supported on W2K and lower */
+        todo_wine
+        ok(ret ||
+           broken(!ret && GetLastError() == NTE_BAD_FLAGS),
+           "CryptImportKey error %08x\n", GetLastError());
 
-    /* Importing a 16-byte key works with the default provider. */
-    SetLastError(0xdeadbeef);
-    ret = pCryptImportKey(provider, (BYTE*)&key_blob,
-                      sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
-                      0, CRYPT_IPSEC_HMAC_KEY, &hkey);
-    /* CRYPT_IPSEC_HMAC_KEY is not supported on W2K and lower */
-    todo_wine
-    ok(ret ||
-       broken(!ret && GetLastError() == NTE_BAD_FLAGS),
-       "CryptImportKey error %08x\n", GetLastError());
-
-    pCryptDestroyKey(hkey);
-    pCryptReleaseContext(provider, 0);
+        if (ret)
+            pCryptDestroyKey(hkey);
+        pCryptReleaseContext(provider, 0);
+    }
 
     SetLastError(0xdeadbeef);
     ret = pCryptAcquireContextA(&provider, NULL, MS_DEF_PROV,
                                 PROV_RSA_FULL, CRYPT_VERIFYCONTEXT);
     ok(ret, "CryptAcquireContext error %08x\n", GetLastError());
 
-    /* Importing a 16-byte key doesn't work with the base provider.. */
-    SetLastError(0xdeadbeef);
-    ret = pCryptImportKey(provider, (BYTE*)&key_blob,
-                          sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
-                          0, 0, &hkey);
-    ok(!ret && (GetLastError() == NTE_BAD_DATA ||
-                GetLastError() == NTE_BAD_LEN || /* Win7 */
-                GetLastError() == NTE_BAD_TYPE || /* W2K */
-                GetLastError() == NTE_PERM), /* Win9x, WinMe and NT4 */
-       "unexpected error %08x\n", GetLastError());
-    /* but importing an 56-bit (7-byte) key does.. */
-    key_blob.key_size = 7;
-    SetLastError(0xdeadbeef);
-    ret = pCryptImportKey(provider, (BYTE*)&key_blob,
-                          sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
-                          0, 0, &hkey);
-    ok(ret ||
-       broken(!ret && GetLastError() == NTE_BAD_TYPE) || /* W2K */
-       broken(!ret && GetLastError() == NTE_PERM), /* Win9x, WinMe and NT4 */
-       "CryptAcquireContext error %08x\n", GetLastError());
-    pCryptDestroyKey(hkey);
-    /* as does importing a 16-byte key with the base provider when
-     * CRYPT_IPSEC_HMAC_KEY is specified.
-     */
-    key_blob.key_size = sizeof(key);
-    SetLastError(0xdeadbeef);
-    ret = pCryptImportKey(provider, (BYTE*)&key_blob,
-                          sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
-                          0, CRYPT_IPSEC_HMAC_KEY, &hkey);
-    /* CRYPT_IPSEC_HMAC_KEY is not supported on W2K and lower */
-    todo_wine
-    ok(ret ||
-       broken(!ret && GetLastError() == NTE_BAD_FLAGS),
-       "CryptImportKey error %08x\n", GetLastError());
-    pCryptDestroyKey(hkey);
+    if (ret)
+    {
+        /* Importing a 16-byte key doesn't work with the base provider.. */
+        SetLastError(0xdeadbeef);
+        ret = pCryptImportKey(provider, (BYTE*)&key_blob,
+                              sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
+                              0, 0, &hkey);
+        ok(!ret && (GetLastError() == NTE_BAD_DATA ||
+                    GetLastError() == NTE_BAD_LEN || /* Win7 */
+                    GetLastError() == NTE_BAD_TYPE || /* W2K */
+                    GetLastError() == NTE_PERM), /* Win9x, WinMe and NT4 */
+           "unexpected error %08x\n", GetLastError());
+        /* but importing an 56-bit (7-byte) key does.. */
+        key_blob.key_size = 7;
+        SetLastError(0xdeadbeef);
+        ret = pCryptImportKey(provider, (BYTE*)&key_blob,
+                              sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
+                              0, 0, &hkey);
+        ok(ret ||
+           broken(!ret && GetLastError() == NTE_BAD_TYPE) || /* W2K */
+           broken(!ret && GetLastError() == NTE_PERM), /* Win9x, WinMe and NT4 */
+           "CryptAcquireContext error %08x\n", GetLastError());
+        if (ret)
+            pCryptDestroyKey(hkey);
+        /* as does importing a 16-byte key with the base provider when
+         * CRYPT_IPSEC_HMAC_KEY is specified.
+         */
+        key_blob.key_size = sizeof(key);
+        SetLastError(0xdeadbeef);
+        ret = pCryptImportKey(provider, (BYTE*)&key_blob,
+                              sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
+                              0, CRYPT_IPSEC_HMAC_KEY, &hkey);
+        /* CRYPT_IPSEC_HMAC_KEY is not supported on W2K and lower */
+        todo_wine
+        ok(ret ||
+           broken(!ret && GetLastError() == NTE_BAD_FLAGS),
+           "CryptImportKey error %08x\n", GetLastError());
+        if (ret)
+            pCryptDestroyKey(hkey);
 
-    pCryptReleaseContext(provider, 0);
+        pCryptReleaseContext(provider, 0);
+    }
 
     key_blob.key_size = sizeof(key);
     SetLastError(0xdeadbeef);
@@ -1027,34 +1038,39 @@ static void test_rc2_keylen(void)
                                 PROV_RSA_FULL, CRYPT_VERIFYCONTEXT);
     ok(ret, "CryptAcquireContext error %08x\n", GetLastError());
 
-    /* Importing a 16-byte key also works with the default provider when
-     * CRYPT_IPSEC_HMAC_KEY is specified.
-     */
-    SetLastError(0xdeadbeef);
-    ret = pCryptImportKey(provider, (BYTE*)&key_blob,
-                          sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
-                          0, CRYPT_IPSEC_HMAC_KEY, &hkey);
-    todo_wine
-    ok(ret ||
-       broken(!ret && GetLastError() == NTE_BAD_FLAGS),
-       "CryptImportKey error %08x\n", GetLastError());
-    pCryptDestroyKey(hkey);
+    if (ret)
+    {
+        /* Importing a 16-byte key also works with the default provider when
+         * CRYPT_IPSEC_HMAC_KEY is specified.
+         */
+        SetLastError(0xdeadbeef);
+        ret = pCryptImportKey(provider, (BYTE*)&key_blob,
+                              sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
+                              0, CRYPT_IPSEC_HMAC_KEY, &hkey);
+        todo_wine
+        ok(ret ||
+           broken(!ret && GetLastError() == NTE_BAD_FLAGS),
+           "CryptImportKey error %08x\n", GetLastError());
+        if (ret)
+            pCryptDestroyKey(hkey);
 
-    /* There is no apparent limit to the size of the input key when
-     * CRYPT_IPSEC_HMAC_KEY is specified.
-     */
-    key_blob.key_size = sizeof(key_blob.key_data);
-    SetLastError(0xdeadbeef);
-    ret = pCryptImportKey(provider, (BYTE*)&key_blob,
-                          sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
-                          0, CRYPT_IPSEC_HMAC_KEY, &hkey);
-    todo_wine
-    ok(ret ||
-       broken(!ret && GetLastError() == NTE_BAD_FLAGS),
-       "CryptImportKey error %08x\n", GetLastError());
-    pCryptDestroyKey(hkey);
+        /* There is no apparent limit to the size of the input key when
+         * CRYPT_IPSEC_HMAC_KEY is specified.
+         */
+        key_blob.key_size = sizeof(key_blob.key_data);
+        SetLastError(0xdeadbeef);
+        ret = pCryptImportKey(provider, (BYTE*)&key_blob,
+                              sizeof(BLOBHEADER)+sizeof(DWORD)+key_blob.key_size,
+                              0, CRYPT_IPSEC_HMAC_KEY, &hkey);
+        todo_wine
+        ok(ret ||
+           broken(!ret && GetLastError() == NTE_BAD_FLAGS),
+           "CryptImportKey error %08x\n", GetLastError());
+        if (ret)
+            pCryptDestroyKey(hkey);
 
-    pCryptReleaseContext(provider, 0);
+        pCryptReleaseContext(provider, 0);
+    }
 }
 
 START_TEST(crypt)
