@@ -226,11 +226,6 @@ HRESULT volume_init(IDirect3DVolume8Impl *volume, IDirect3DDevice8Impl *device,
 /* IDirect3DSwapChain8 */
 /* ------------------- */
 
-/*****************************************************************************
- * Predeclare the interface implementation structures
- */
-extern const IDirect3DSwapChain8Vtbl Direct3DSwapChain8_Vtbl DECLSPEC_HIDDEN;
-
 /*****************************************************************************
  * IDirect3DSwapChain8 implementation structure
  */
@@ -247,6 +242,9 @@ struct IDirect3DSwapChain8Impl
     LPDIRECT3DDEVICE8              parentDevice;
 };
 
+HRESULT swapchain_init(IDirect3DSwapChain8Impl *swapchain, IDirect3DDevice8Impl *device,
+        D3DPRESENT_PARAMETERS *present_parameters) DECLSPEC_HIDDEN;
+
 /* ----------------- */
 /* IDirect3DSurface8 */
 /* ----------------- */
