@@ -408,7 +408,7 @@ static HRESULT JSGlobal_eval(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DIS
         return throw_syntax_error(ctx, ei, hres, NULL);
     }
 
-    hres = exec_source(ctx->exec_ctx, parser_ctx, parser_ctx->source, ei, retv);
+    hres = exec_source(ctx->exec_ctx, parser_ctx, parser_ctx->source, EXECT_EVAL, ei, retv);
     parser_release(parser_ctx);
 
     return hres;
@@ -811,8 +811,7 @@ static HRESULT JSGlobal_encodeURI(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
             i = WideCharToMultiByte(CP_UTF8, 0, ptr, 1, NULL, 0, NULL, NULL)*3;
             if(!i) {
                 SysFreeString(str);
-                FIXME("throw URIError\n");
-                return E_FAIL;
+                return throw_uri_error(ctx, ei, IDS_URI_INVALID_CHAR, NULL);
             }
 
             len += i;
@@ -931,11 +930,127 @@ static HRESULT JSGlobal_encodeURIComponent(script_ctx_t *ctx, vdisp_t *jsthis, W
     return S_OK;
 }
 
+/* ECMA-262 3rd Edition    15.1.3.2 */
 static HRESULT JSGlobal_decodeURIComponent(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
         VARIANT *retv, jsexcept_t *ei, IServiceProvider *sp)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    BSTR str, ret;
+    const WCHAR *ptr;
+    WCHAR *out_ptr;
+    DWORD len = 0;
+    HRESULT hres;
+
+    TRACE("\n");
+
+    if(!arg_cnt(dp)) {
+        if(retv) {
+            ret = SysAllocString(undefinedW);
+            if(!ret)
+                return E_OUTOFMEMORY;
+
+            V_VT(retv) = VT_BSTR;
+            V_BSTR(retv) = ret;
+        }
+
+        return S_OK;
+    }
+
+    hres = to_string(ctx, get_arg(dp, 0), ei, &str);
+    if(FAILED(hres))
+        return hres;
+
+    ptr = str;
+    while(*ptr) {
+        if(*ptr == '%') {
+            char octets[4];
+            unsigned char mask = 0x80;
+            int i, size, num_bytes = 0;
+            if(hex_to_int(*(ptr+1)) < 0 || hex_to_int(*(ptr+2)) < 0) {
+                FIXME("Throw URIError: Invalid hex sequence\n");
+                SysFreeString(str);
+                return E_FAIL;
+            }
+            octets[0] = (hex_to_int(*(ptr+1)) << 4) + hex_to_int(*(ptr+2));
+            ptr += 3;
+            while(octets[0] & mask) {
+                mask = mask >> 1;
+                ++num_bytes;
+            }
+            if(num_bytes == 1 || num_bytes > 4) {
+                FIXME("Throw URIError: Invalid initial UTF character\n");
+                SysFreeString(str);
+                return E_FAIL;
+            }
+            for(i = 1; i < num_bytes; ++i) {
+                if(*ptr != '%'){
+                    FIXME("Throw URIError: Incomplete UTF sequence\n");
+                    SysFreeString(str);
+                    return E_FAIL;
+                }
+                if(hex_to_int(*(ptr+1)) < 0 || hex_to_int(*(ptr+2)) < 0) {
+                    FIXME("Throw URIError: Invalid hex sequence\n");
+                    SysFreeString(str);
+                    return E_FAIL;
+                }
+                octets[i] = (hex_to_int(*(ptr+1)) << 4) + hex_to_int(*(ptr+2));
+                ptr += 3;
+            }
+            size = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, octets,
+                    num_bytes ? num_bytes : 1, NULL, 0);
+            if(size == 0) {
+                FIXME("Throw URIError: Invalid UTF sequence\n");
+                SysFreeString(str);
+                return E_FAIL;
+            }
+            len += size;
+        }else {
+            ++ptr;
+            ++len;
+        }
+    }
+
+    out_ptr = ret = SysAllocStringLen(NULL, len);
+    if(!ret) {
+        SysFreeString(str);
+        return E_OUTOFMEMORY;
+    }
+
+    ptr = str;
+    while(*ptr) {
+        if(*ptr == '%') {
+            char octets[4];
+            unsigned char mask = 0x80;
+            int i, size, num_bytes = 0;
+            octets[0] = (hex_to_int(*(ptr+1)) << 4) + hex_to_int(*(ptr+2));
+            ptr += 3;
+            while(octets[0] & mask) {
+                mask = mask >> 1;
+                ++num_bytes;
+            }
+            for(i = 1; i < num_bytes; ++i) {
+                octets[i] = (hex_to_int(*(ptr+1)) << 4) + hex_to_int(*(ptr+2));
+                ptr += 3;
+            }
+            size = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, octets,
+                    num_bytes ? num_bytes : 1, out_ptr, len);
+            len -= size;
+            out_ptr += size;
+        }else {
+            *out_ptr++ = *ptr++;
+            --len;
+        }
+    }
+
+    SysFreeString(str);
+
+    if(retv) {
+        V_VT(retv) = VT_BSTR;
+        V_BSTR(retv) = ret;
+    }else {
+        SysFreeString(ret);
+    }
+
+    return S_OK;
 }
 
 static const builtin_prop_t JSGlobal_props[] = {
@@ -1061,6 +1176,11 @@ HRESULT init_global(script_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
+    V_VT(&var) = VT_EMPTY;
+    hres = jsdisp_propput_name(ctx->global, undefinedW, &var, NULL/*FIXME*/, NULL/*FIXME*/);
+    if(FAILED(hres))
+        return hres;
+
     V_VT(&var) = VT_DISPATCH;
     V_DISPATCH(&var) = (IDispatch*)_IDispatchEx_(math);
     hres = jsdisp_propput_name(ctx->global, MathW, &var, NULL/*FIXME*/, NULL/*FIXME*/);
