@@ -63,40 +63,97 @@ static const WCHAR yellowW[] = {'y','e','l','l','o','w',0};
 
 static const struct {
     LPCWSTR keyword;
-    const WCHAR hexstr[8];
+    DWORD rgb;
 } keyword_table[] = {
-    {aquaW,     {'#','0','0','f','f','f','f',0}},
-    {blackW,    {'#','0','0','0','0','0','0',0}},
-    {blueW,     {'#','0','0','0','0','f','f',0}},
-    {fuchsiaW,  {'#','f','f','0','0','f','f',0}},
-    {grayW,     {'#','8','0','8','0','8','0',0}},
-    {greenW,    {'#','0','0','8','0','0','0',0}},
-    {limeW,     {'#','0','0','f','f','0','0',0}},
-    {maroonW,   {'#','8','0','0','0','0','0',0}},
-    {navyW,     {'#','0','0','0','0','8','0',0}},
-    {oliveW,    {'#','8','0','8','0','0','0',0}},
-    {purpleW,   {'#','8','0','0','0','8','0',0}},
-    {redW,      {'#','f','f','0','0','0','0',0}},
-    {silverW,   {'#','c','0','c','0','c','0',0}},
-    {tealW,     {'#','0','0','8','0','8','0',0}},
-    {whiteW,    {'#','f','f','f','f','f','f',0}},
-    {yellowW,   {'#','f','f','f','f','0','0',0}}
+    {aquaW,     0x00ffff},
+    {blackW,    0x000000},
+    {blueW,     0x0000ff},
+    {fuchsiaW,  0xff00ff},
+    {grayW,     0x808080},
+    {greenW,    0x008000},
+    {limeW,     0x00ff00},
+    {maroonW,   0x800000},
+    {navyW,     0x000080},
+    {oliveW,    0x808000},
+    {purpleW,   0x800080},
+    {redW,      0xff0000},
+    {silverW,   0xc0c0c0},
+    {tealW,     0x008080},
+    {whiteW,    0xffffff},
+    {yellowW,   0xffff00}
 };
 
-static BSTR nscolor_to_str(LPCWSTR color)
+static int comp_value(const WCHAR *ptr, int dpc)
 {
-    int i;
+    int ret = 0;
+    WCHAR ch;
 
-    if(!color || *color == '#')
-        return SysAllocString(color);
+    if(dpc > 2)
+        dpc = 2;
 
-    for(i=0; i < sizeof(keyword_table)/sizeof(keyword_table[0]); i++) {
-        if(!strcmpiW(color, keyword_table[i].keyword))
-            return SysAllocString(keyword_table[i].hexstr);
+    while(dpc--) {
+        if(!*ptr)
+            ret *= 16;
+        else if(isdigitW(ch = *ptr++))
+            ret = ret*16 + (ch-'0');
+        else if('a' <= ch && ch <= 'f')
+            ret = ret*16 + (ch-'a') + 10;
+        else if('A' <= ch && ch <= 'F')
+            ret = ret*16 + (ch-'A') + 10;
+        else
+            ret *= 16;
     }
 
-    WARN("unknown color %s\n", debugstr_w(color));
-    return SysAllocString(color);
+    return ret;
+}
+
+/* Based on Gecko NS_LooseHexToRGB */
+static int loose_hex_to_rgb(const WCHAR *hex)
+{
+    int len, dpc;
+
+    len = strlenW(hex);
+    if(*hex == '#') {
+        hex++;
+        len--;
+    }
+    if(len <= 3)
+        return 0;
+
+    dpc = min(len/3 + (len%3 ? 1 : 0), 4);
+    return (comp_value(hex, dpc) << 16)
+        | (comp_value(hex+dpc, dpc) << 8)
+        | comp_value(hex+2*dpc, dpc);
+}
+
+static HRESULT nscolor_to_str(LPCWSTR color, BSTR *ret)
+{
+    int i, rgb = -1;
+
+    static const WCHAR formatW[] = {'#','%','0','2','x','%','0','2','x','%','0','2','x',0};
+
+    if(!color || !*color) {
+        *ret = NULL;
+        return S_OK;
+    }
+
+    if(*color != '#') {
+        for(i=0; i < sizeof(keyword_table)/sizeof(keyword_table[0]); i++) {
+            if(!strcmpiW(color, keyword_table[i].keyword))
+                rgb = keyword_table[i].rgb;
+        }
+    }
+    if(rgb == -1)
+        rgb = loose_hex_to_rgb(color);
+
+    *ret = SysAllocStringLen(NULL, 7);
+    if(!*ret)
+        return E_OUTOFMEMORY;
+
+    sprintfW(*ret, formatW, rgb>>16, (rgb>>8)&0xff, rgb&0xff);
+
+    TRACE("%s -> %s\n", debugstr_w(color), debugstr_w(*ret));
+    return S_OK;
 }
 
 static BOOL variant_to_nscolor(const VARIANT *v, nsAString *nsstr)
@@ -348,23 +405,25 @@ static HRESULT WINAPI HTMLBodyElement_get_bgColor(IHTMLBodyElement *iface, VARIA
     HTMLBodyElement *This = HTMLBODY_THIS(iface);
     nsAString strColor;
     nsresult nsres;
-    const PRUnichar *color;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
     nsAString_Init(&strColor, NULL);
     nsres = nsIDOMHTMLBodyElement_GetBgColor(This->nsbody, &strColor);
-    if(NS_FAILED(nsres))
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *color;
+
+        nsAString_GetData(&strColor, &color);
+        V_VT(p) = VT_BSTR;
+        hres = nscolor_to_str(color, &V_BSTR(p));
+    }else {
         ERR("SetBgColor failed: %08x\n", nsres);
-
-    nsAString_GetData(&strColor, &color);
-
-    V_VT(p) = VT_BSTR;
-    V_BSTR(p) = nscolor_to_str(color);
+        hres = E_FAIL;
+    }
 
     nsAString_Finish(&strColor);
-
-    return S_OK;
+    return hres;
 }
 
 static HRESULT WINAPI HTMLBodyElement_put_text(IHTMLBodyElement *iface, VARIANT v)
@@ -389,22 +448,21 @@ static HRESULT WINAPI HTMLBodyElement_get_text(IHTMLBodyElement *iface, VARIANT
     HTMLBodyElement *This = HTMLBODY_THIS(iface);
     nsAString text;
     nsresult nsres;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
     nsAString_Init(&text, NULL);
-
-    V_VT(p) = VT_BSTR;
-    V_BSTR(p) = NULL;
-
     nsres = nsIDOMHTMLBodyElement_GetText(This->nsbody, &text);
-    if(NS_SUCCEEDED(nsres))
-    {
-        const PRUnichar *sText;
-        nsAString_GetData(&text, &sText);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *color;
 
+        nsAString_GetData(&text, &color);
         V_VT(p) = VT_BSTR;
-        V_BSTR(p) = SysAllocString(sText);
+        hres = nscolor_to_str(color, &V_BSTR(p));
+    }else {
+        ERR("GetText failed: %08x\n", nsres);
+        hres = E_FAIL;
     }
 
     nsAString_Finish(&text);
@@ -725,8 +783,8 @@ static event_target_t **HTMLBodyElement_get_event_target(HTMLDOMNode *iface)
 {
     HTMLBodyElement *This = HTMLBODY_NODE_THIS(iface);
 
-    return This->textcont.element.node.doc && This->textcont.element.node.doc->basedoc.window
-        ? &This->textcont.element.node.doc->basedoc.window->event_target
+    return This->textcont.element.node.doc
+        ? &This->textcont.element.node.doc->body_event_target
         : &This->textcont.element.node.event_target;
 }
 
