@@ -27,6 +27,7 @@
 #include "winuser.h"
 #include "winreg.h"
 #include "ole2.h"
+#include "shlguid.h"
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
@@ -36,8 +37,9 @@
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
 enum {
+    MUTATION_BINDTOTREE,
     MUTATION_COMMENT,
-    MUTATION_IFRAME,
+    MUTATION_ENDLOAD,
     MUTATION_SCRIPT
 };
 
@@ -230,6 +232,28 @@ static nsrefcnt NSAPI nsRunnable_Release(nsIRunnable *iface)
     return htmldoc_release(&This->basedoc);
 }
 
+static void push_mutation_queue(HTMLDocumentNode *doc, DWORD type, nsISupports *nsiface)
+{
+    mutation_queue_t *elem;
+
+    elem = heap_alloc(sizeof(mutation_queue_t));
+    if(!elem)
+        return;
+
+    elem->next = NULL;
+    elem->type = type;
+    elem->nsiface = nsiface;
+    if(nsiface)
+        nsISupports_AddRef(nsiface);
+
+    if(doc->mutation_queue_tail) {
+        doc->mutation_queue_tail = doc->mutation_queue_tail->next = elem;
+    }else {
+        doc->mutation_queue = doc->mutation_queue_tail = elem;
+        add_script_runner(doc);
+    }
+}
+
 static void pop_mutation_queue(HTMLDocumentNode *doc)
 {
     mutation_queue_t *tmp = doc->mutation_queue;
@@ -241,46 +265,93 @@ static void pop_mutation_queue(HTMLDocumentNode *doc)
     if(!tmp->next)
         doc->mutation_queue_tail = NULL;
 
-    nsISupports_Release(tmp->nsiface);
+    if(tmp->nsiface)
+        nsISupports_Release(tmp->nsiface);
     heap_free(tmp);
 }
 
-static nsresult init_iframe_window(HTMLDocumentNode *doc, nsISupports *nsunk)
+static void bind_to_tree(HTMLDocumentNode *doc, nsISupports *nsiface)
 {
-    nsIDOMHTMLIFrameElement *nsiframe;
-    nsIDOMWindow *nswindow;
-    nsIDOMDocument *nsdoc;
+    nsIDOMNode *nsnode;
+    HTMLDOMNode *node;
     nsresult nsres;
 
-    nsres = nsISupports_QueryInterface(nsunk, &IID_nsIDOMHTMLIFrameElement, (void**)&nsiframe);
-    if(NS_FAILED(nsres)) {
-        ERR("Could not get nsIDOMHTMLIFrameElement: %08x\n", nsres);
-        return nsres;
+    nsres = nsISupports_QueryInterface(nsiface, &IID_nsIDOMNode, (void**)&nsnode);
+    if(NS_FAILED(nsres))
+        return;
+
+    node = get_node(doc, nsnode, TRUE);
+    nsIDOMNode_Release(nsnode);
+    if(!node) {
+        ERR("Could not get node\n");
+        return;
     }
 
-    nsres = nsIDOMHTMLIFrameElement_GetContentDocument(nsiframe, &nsdoc);
-    nsIDOMHTMLIFrameElement_Release(nsiframe);
-    if(NS_FAILED(nsres) || !nsdoc) {
-        ERR("GetContentDocument failed: %08x\n", nsres);
-        return nsres;
+    if(node->vtbl->bind_to_tree)
+        node->vtbl->bind_to_tree(node);
+}
+
+/* Calls undocumented 69 cmd of CGID_Explorer */
+static void call_explorer_69(HTMLDocumentObj *doc)
+{
+    IOleCommandTarget *olecmd;
+    VARIANT var;
+    HRESULT hres;
+
+    if(!doc->client)
+        return;
+
+    hres = IOleClientSite_QueryInterface(doc->client, &IID_IOleCommandTarget, (void**)&olecmd);
+    if(FAILED(hres))
+        return;
+
+    VariantInit(&var);
+    hres = IOleCommandTarget_Exec(olecmd, &CGID_Explorer, 69, 0, NULL, &var);
+    IOleCommandTarget_Release(olecmd);
+    if(SUCCEEDED(hres) && V_VT(&var) != VT_NULL)
+        FIXME("handle result\n");
+}
+
+static void parse_complete_proc(task_t *task)
+{
+    HTMLDocumentObj *doc = ((docobj_task_t*)task)->doc;
+
+    TRACE("(%p)\n", doc);
+
+    if(doc->usermode == EDITMODE)
+        init_editor(&doc->basedoc);
+
+    call_explorer_69(doc);
+    call_property_onchanged(&doc->basedoc.cp_propnotif, 1005);
+    call_explorer_69(doc);
+
+    /* FIXME: IE7 calls EnableModelless(TRUE), EnableModelless(FALSE) and sets interactive state here */
+
+    set_ready_state(doc->basedoc.window, READYSTATE_INTERACTIVE);
+}
+
+static void handle_end_load(HTMLDocumentNode *This)
+{
+    docobj_task_t *task;
+
+    TRACE("\n");
+
+    if(This != This->basedoc.doc_obj->basedoc.doc_node) {
+        set_ready_state(This->basedoc.window, READYSTATE_INTERACTIVE);
+        return;
     }
 
-    nswindow = get_nsdoc_window(nsdoc);
-    nsIDOMDocument_Release(nsdoc);
-    if(!nswindow)
-        return NS_ERROR_FAILURE;
+    task = heap_alloc(sizeof(docobj_task_t));
+    if(!task)
+        return;
 
-    if(!nswindow_to_window(nswindow)) {
-        HTMLWindow *window;
-        HRESULT hres;
+    task->doc = This->basedoc.doc_obj;
 
-        hres = HTMLWindow_Create(doc->basedoc.doc_obj, nswindow, doc->basedoc.window, &window);
-        if(SUCCEEDED(hres))
-            IHTMLWindow2_Release(HTMLWINDOW2(window));
-    }
-
-    nsIDOMWindow_Release(nswindow);
-    return NS_OK;
+    /*
+     * This should be done in the worker thread that parses HTML,
+     * but we don't have such thread (Gecko parses HTML for us).
+     */
+    push_task(&task->header, &parse_complete_proc, This->basedoc.doc_obj->basedoc.task_magic);
 }
 
 static nsresult NSAPI nsRunnable_Run(nsIRunnable *iface)
@@ -292,6 +363,10 @@ static nsresult NSAPI nsRunnable_Run(nsIRunnable *iface)
 
     while(This->mutation_queue) {
         switch(This->mutation_queue->type) {
+        case MUTATION_BINDTOTREE:
+            bind_to_tree(This, This->mutation_queue->nsiface);
+            break;
+
         case MUTATION_COMMENT: {
             nsIDOMComment *nscomment;
             nsAString comment_str;
@@ -339,8 +414,8 @@ static nsresult NSAPI nsRunnable_Run(nsIRunnable *iface)
             break;
         }
 
-        case MUTATION_IFRAME:
-            init_iframe_window(This, This->mutation_queue->nsiface);
+        case MUTATION_ENDLOAD:
+            handle_end_load(This);
             break;
 
         case MUTATION_SCRIPT: {
@@ -479,21 +554,7 @@ static void NSAPI nsDocumentObserver_EndLoad(nsIDocumentObserver *iface, nsIDocu
     TRACE("\n");
 
     This->content_ready = TRUE;
-
-    if(This == This->basedoc.doc_obj->basedoc.doc_node) {
-        task_t *task;
-        task = heap_alloc(sizeof(task_t));
-
-        task->doc = &This->basedoc.doc_obj->basedoc;
-        task->task_id = TASK_PARSECOMPLETE;
-        task->next = NULL;
-
-        /*
-         * This should be done in the worker thread that parses HTML,
-         * but we don't have such thread (Gecko parses HTML for us).
-         */
-        push_task(task);
-    }
+    push_mutation_queue(This, MUTATION_ENDLOAD, NULL);
 }
 
 static void NSAPI nsDocumentObserver_ContentStatesChanged(nsIDocumentObserver *iface, nsIDocument *aDocument,
@@ -531,30 +592,12 @@ static void NSAPI nsDocumentObserver_StyleRuleRemoved(nsIDocumentObserver *iface
 {
 }
 
-static void push_mutation_queue(HTMLDocumentNode *doc, DWORD type, nsISupports *nsiface)
-{
-    mutation_queue_t *elem;
-
-    elem = heap_alloc(sizeof(mutation_queue_t));
-    if(!elem)
-        return;
-
-    elem->next = NULL;
-    elem->type = type;
-    elem->nsiface = nsiface;
-    nsISupports_AddRef(nsiface);
-
-    if(doc->mutation_queue_tail)
-        doc->mutation_queue_tail = doc->mutation_queue_tail->next = elem;
-    else
-        doc->mutation_queue = doc->mutation_queue_tail = elem;
-}
-
 static void NSAPI nsDocumentObserver_BindToDocument(nsIDocumentObserver *iface, nsIDocument *aDocument,
         nsIContent *aContent)
 {
     HTMLDocumentNode *This = NSDOCOBS_THIS(iface);
     nsIDOMHTMLIFrameElement *nsiframe;
+    nsIDOMHTMLFrameElement *nsframe;
     nsIDOMComment *nscomment;
     nsIDOMElement *nselem;
     nsresult nsres;
@@ -573,16 +616,22 @@ static void NSAPI nsDocumentObserver_BindToDocument(nsIDocumentObserver *iface,
 
         push_mutation_queue(This, MUTATION_COMMENT, (nsISupports*)nscomment);
         nsIDOMComment_Release(nscomment);
-        add_script_runner(This);
     }
 
     nsres = nsISupports_QueryInterface(aContent, &IID_nsIDOMHTMLIFrameElement, (void**)&nsiframe);
     if(NS_SUCCEEDED(nsres)) {
         TRACE("iframe node\n");
 
-        push_mutation_queue(This, MUTATION_IFRAME, (nsISupports*)nsiframe);
+        push_mutation_queue(This, MUTATION_BINDTOTREE, (nsISupports*)nsiframe);
         nsIDOMHTMLIFrameElement_Release(nsiframe);
-        add_script_runner(This);
+    }
+
+    nsres = nsISupports_QueryInterface(aContent, &IID_nsIDOMHTMLFrameElement, (void**)&nsframe);
+    if(NS_SUCCEEDED(nsres)) {
+        TRACE("frame node\n");
+
+        push_mutation_queue(This, MUTATION_BINDTOTREE, (nsISupports*)nsframe);
+        nsIDOMHTMLFrameElement_Release(nsframe);
     }
 }
 
@@ -601,7 +650,6 @@ static void NSAPI nsDocumentObserver_DoneAddingChildren(nsIDocumentObserver *ifa
 
         push_mutation_queue(This, MUTATION_SCRIPT, (nsISupports*)nsscript);
         nsIDOMHTMLScriptElement_Release(nsscript);
-        add_script_runner(This);
     }
 }
 
