@@ -40,6 +40,7 @@
 WINE_DEFAULT_DEBUG_CHANNEL(ntoskrnl);
 WINE_DECLARE_DEBUG_CHANNEL(relay);
 
+extern LONG CALLBACK vectored_handler( EXCEPTION_POINTERS *ptrs );
 
 KSYSTEM_TIME KeTickCount = { 0, 0, 0 };
 
@@ -121,27 +122,6 @@ static HANDLE get_device_manager(void)
     return ret;
 }
 
-/* exception handler for emulation of privileged instructions */
-static LONG CALLBACK vectored_handler( EXCEPTION_POINTERS *ptrs )
-{
-    EXCEPTION_RECORD *record = ptrs->ExceptionRecord;
-
-    if (record->ExceptionCode == EXCEPTION_ACCESS_VIOLATION ||
-        record->ExceptionCode == EXCEPTION_PRIV_INSTRUCTION)
-    {
-#ifdef __i386__
-        CONTEXT *context = ptrs->ContextRecord;
-        extern DWORD __wine_emulate_instruction( EXCEPTION_RECORD *rec, CONTEXT *context );
-
-        if (__wine_emulate_instruction( record, context ) == ExceptionContinueExecution)
-            return EXCEPTION_CONTINUE_EXECUTION;
-#else
-        FIXME( "Privileged instruction emulation not implemented on this CPU\n" );
-#endif
-    }
-    return EXCEPTION_CONTINUE_SEARCH;
-}
-
 /* process an ioctl request for a given device */
 static NTSTATUS process_ioctl( DEVICE_OBJECT *device, ULONG code, void *in_buff, ULONG in_size,
                                void *out_buff, ULONG *out_size )
@@ -373,6 +353,16 @@ void WINAPI IoFreeIrp( IRP *irp )
 }
 
 
+/***********************************************************************
+ *           IoAllocateErrorLogEntry  (NTOSKRNL.EXE.@)
+ */
+PVOID WINAPI IoAllocateErrorLogEntry( PVOID IoObject, UCHAR EntrySize )
+{
+    FIXME( "stub: %p, %u\n", IoObject, EntrySize );
+    return NULL;
+}
+
+
 /***********************************************************************
  *           IoAllocateMdl  (NTOSKRNL.EXE.@)
  */
@@ -970,6 +960,15 @@ NTSTATUS WINAPI FsRtlRegisterUncProvider(PHANDLE MupHandle, PUNICODE_STRING Redi
     return STATUS_NOT_IMPLEMENTED;
 }
 
+/***********************************************************************
+ *           KeGetCurrentThread / PsGetCurrentThread   (NTOSKRNL.EXE.@)
+ */
+PRKTHREAD WINAPI KeGetCurrentThread(void)
+{
+    FIXME("() stub\n");
+    return NULL;
+}
+
 /***********************************************************************
  *           KeInitializeEvent   (NTOSKRNL.EXE.@)
  */
@@ -988,6 +987,15 @@ void WINAPI KeInitializeMutex(PRKMUTEX Mutex, ULONG Level)
 }
 
 
+/***********************************************************************
+ *           KeInitializeSemaphore   (NTOSKRNL.EXE.@)
+ */
+void WINAPI KeInitializeSemaphore( PRKSEMAPHORE Semaphore, LONG Count, LONG Limit )
+{
+    FIXME( "(%p %d %d) stub\n", Semaphore , Count, Limit );
+}
+
+
 /***********************************************************************
  *           KeInitializeSpinLock   (NTOSKRNL.EXE.@)
  */
@@ -1070,6 +1078,17 @@ void WINAPI KeQueryTickCount( LARGE_INTEGER *count )
 }
 
 
+/***********************************************************************
+ *           KeReleaseSemaphore   (NTOSKRNL.EXE.@)
+ */
+LONG WINAPI KeReleaseSemaphore( PRKSEMAPHORE Semaphore, KPRIORITY Increment,
+                                LONG Adjustment, BOOLEAN Wait )
+{
+    FIXME("(%p %d %d %d) stub\n", Semaphore, Increment, Adjustment, Wait );
+    return 0;
+}
+
+
 /***********************************************************************
  *           KeQueryTimeIncrement   (NTOSKRNL.EXE.@)
  */
@@ -1079,6 +1098,16 @@ ULONG WINAPI KeQueryTimeIncrement(void)
 }
 
 
+/***********************************************************************
+ *           KeSetPriorityThread   (NTOSKRNL.EXE.@)
+ */
+KPRIORITY WINAPI KeSetPriorityThread( PKTHREAD Thread, KPRIORITY Priority )
+{
+    FIXME("(%p %d)\n", Thread, Priority);
+    return Priority;
+}
+
+
 /***********************************************************************
  *           KeWaitForSingleObject   (NTOSKRNL.EXE.@)
  */
@@ -1359,7 +1388,9 @@ BOOL WINAPI DllMain( HINSTANCE inst, DWORD reason, LPVOID reserved )
     {
     case DLL_PROCESS_ATTACH:
         DisableThreadLibraryCalls( inst );
+#ifdef __i386__
         handler = RtlAddVectoredExceptionHandler( TRUE, vectored_handler );
+#endif
         KeQueryTickCount( &count );  /* initialize the global KeTickCount */
         break;
     case DLL_PROCESS_DETACH:
