@@ -190,6 +190,9 @@ HRESULT WINAPI OleInitialize(LPVOID reserved)
   if (FAILED(hr))
     return hr;
 
+  if (!COM_CurrentInfo()->ole_inits)
+    hr = S_OK;
+
   /*
    * Then, it has to initialize the OLE specific modules.
    * This includes:
@@ -1140,8 +1143,8 @@ HRESULT WINAPI OleLockRunning(LPUNKNOWN pUnknown, BOOL fLock, BOOL fLastUnlockCl
 
     return hres;
   }
-  else
-    return E_INVALIDARG;
+
+  return S_OK;
 }
 
 
