@@ -473,9 +473,16 @@ static void test_inffilelist(void)
     static const char inffile2[] = "test2.inf";
     static const WCHAR inffile2W[] = {'t','e','s','t','2','.','i','n','f',0};
     static const char invalid_inf[] = "invalid.inf";
+    static const WCHAR invalid_infW[] = {'i','n','v','a','l','i','d','.','i','n','f',0};
     static const char *inf =
         "[Version]\n"
         "Signature=\"$Chicago$\"";
+    static const char *inf2 =
+        "[Version]\n"
+        "Signature=\"$CHICAGO$\"";
+    static const char *infNT =
+        "[Version]\n"
+        "Signature=\"$WINDOWS NT$\"";
 
     WCHAR *p, *ptr;
     char dirA[MAX_PATH];
@@ -508,7 +515,6 @@ static void test_inffilelist(void)
     outsize = 0;
     SetLastError(0xdeadbeef);
     ret = pSetupGetInfFileListW(dir, INF_STYLE_WIN4, NULL, 0, &outsize);
-    todo_wine
     ok(!ret, "expected SetupGetInfFileListW to fail!\n");
 
     /* create a private directory, the temp directory may contain some
@@ -521,7 +527,7 @@ static void test_inffilelist(void)
     }
     if (!CreateDirectoryA(dirA, NULL ))
     {
-        win_skip("CreateDirectoryA failed with error %d\n", GetLastError());
+        win_skip("CreateDirectoryA(%s) failed with error %d\n", dirA, GetLastError());
         return;
     }
     if (!SetCurrentDirectoryA(dirA))
@@ -541,7 +547,6 @@ static void test_inffilelist(void)
     ret = pSetupGetInfFileListW(dir, INF_STYLE_WIN4, NULL, 0, &outsize);
     ok(ret, "expected SetupGetInfFileListW to succeed!\n");
     ok(outsize == 1, "expected required buffersize to be 1, got %d\n", outsize);
-    todo_wine
     ok(ERROR_PATH_NOT_FOUND == GetLastError(),
        "expected error ERROR_PATH_NOT_FOUND, got %d\n", GetLastError());
     
@@ -556,9 +561,7 @@ static void test_inffilelist(void)
     outsize = 0xffffffff;
     SetLastError(0xdeadbeef);
     ret = pSetupGetInfFileListW(dir, INF_STYLE_WIN4, NULL, 0, &outsize);
-    todo_wine
     ok(!ret, "expected SetupGetInfFileListW to fail!\n");
-    todo_wine
     ok(ERROR_DIRECTORY == GetLastError(),
        "expected error ERROR_DIRECTORY, got %d\n", GetLastError());
 
@@ -568,9 +571,7 @@ static void test_inffilelist(void)
     dir[lstrlenW(dir)] = '\\';
     SetLastError(0xdeadbeef);
     ret = pSetupGetInfFileListW(dir, INF_STYLE_WIN4, NULL, 0, &outsize);
-    todo_wine
     ok(!ret, "expected SetupGetInfFileListW to fail!\n");
-    todo_wine
     ok(ERROR_DIRECTORY == GetLastError(),
        "expected error ERROR_DIRECTORY, got %d\n", GetLastError());
 
@@ -580,13 +581,57 @@ static void test_inffilelist(void)
     expected = 3 + strlen(inffile) + strlen(inffile2);
     ret = pSetupGetInfFileListW(dir, INF_STYLE_WIN4, buffer, MAX_PATH, &outsize);
     ok(ret, "expected SetupGetInfFileListW to succeed!\n");
-    todo_wine
     ok(expected == outsize, "expected required buffersize to be %d, got %d\n",
          expected, outsize);
     for(p = buffer; lstrlenW(p) && (outsize > (p - buffer)); p+=lstrlenW(p) + 1)
         ok(!lstrcmpW(p,inffile2W) || !lstrcmpW(p,inffileW),
             "unexpected filename %s\n",wine_dbgstr_w(p));
 
+    /* upper case value
+     */
+    create_inf_file(inffile2, inf2);
+    ret = pSetupGetInfFileListW(dir, INF_STYLE_WIN4, buffer, MAX_PATH, &outsize);
+    ok(ret, "expected SetupGetInfFileListW to succeed!\n");
+    ok(expected == outsize, "expected required buffersize to be %d, got %d\n",
+         expected, outsize);
+    for(p = buffer; lstrlenW(p) && (outsize > (p - buffer)); p+=lstrlenW(p) + 1)
+        ok(!lstrcmpW(p,inffile2W) || !lstrcmpW(p,inffileW),
+            "unexpected filename %s\n",wine_dbgstr_w(p));
+
+    /* signature Windows NT is also inf style win4
+     */
+    create_inf_file(inffile2, infNT);
+    expected = 3 + strlen(inffile) + strlen(inffile2);
+    ret = pSetupGetInfFileListW(dir, INF_STYLE_WIN4, buffer, MAX_PATH, &outsize);
+    ok(ret, "expected SetupGetInfFileListW to succeed!\n");
+    ok(expected == outsize, "expected required buffersize to be %d, got %d\n",
+         expected, outsize);
+    for(p = buffer; lstrlenW(p) && (outsize > (p - buffer)); p+=lstrlenW(p) + 1)
+        ok(!lstrcmpW(p,inffile2W) || !lstrcmpW(p,inffileW),
+            "unexpected filename %s\n",wine_dbgstr_w(p));
+
+    /* old style
+     */
+    expected = 2 + strlen(invalid_inf);
+    ret = pSetupGetInfFileListW(dir, INF_STYLE_OLDNT, buffer, MAX_PATH, &outsize);
+    ok(ret, "expected SetupGetInfFileListW to succeed!\n");
+    ok(expected == outsize, "expected required buffersize to be %d, got %d\n",
+         expected, outsize);
+    for(p = buffer; lstrlenW(p) && (outsize > (p - buffer)); p+=lstrlenW(p) + 1)
+        ok(!lstrcmpW(p,invalid_infW), "unexpected filename %s\n",wine_dbgstr_w(p));
+
+    /* mixed style
+     */
+    expected = 4 + strlen(inffile) + strlen(inffile2) + strlen(invalid_inf);
+    ret = pSetupGetInfFileListW(dir, INF_STYLE_OLDNT | INF_STYLE_WIN4, buffer,
+                                MAX_PATH, &outsize);
+    ok(ret, "expected SetupGetInfFileListW to succeed!\n");
+    ok(expected == outsize, "expected required buffersize to be %d, got %d\n",
+         expected, outsize);
+    for(p = buffer; lstrlenW(p) && (outsize > (p - buffer)); p+=lstrlenW(p) + 1)
+        ok(!lstrcmpW(p,inffile2W) || !lstrcmpW(p,inffileW) || !lstrcmpW(p,invalid_infW),
+            "unexpected filename %s\n",wine_dbgstr_w(p));
+
     DeleteFile(inffile);
     DeleteFile(inffile2);
     DeleteFile(invalid_inf);
