@@ -43,6 +43,9 @@
 #endif
 
 /* from pidl.h, not included here: */
+#ifndef PT_CPL             /* Guess, Win7 uses this for CSIDL_CONTROLS */
+#define PT_CPL        0x01 /* no path */
+#endif
 #ifndef PT_GUID
 #define PT_GUID       0x1f /* no path */
 #endif
@@ -93,7 +96,7 @@ static UINT (WINAPI *pGetSystemWow64DirectoryA)(LPSTR,UINT);
 static DLLVERSIONINFO shellVersion = { 0 };
 static LPMALLOC pMalloc;
 static const BYTE guidType[] = { PT_GUID };
-static const BYTE controlPanelType[] = { PT_SHELLEXT, PT_GUID };
+static const BYTE controlPanelType[] = { PT_SHELLEXT, PT_GUID, PT_CPL };
 static const BYTE folderType[] = { PT_FOLDER, PT_FOLDERW };
 static const BYTE favoritesType[] = { PT_FOLDER, PT_FOLDERW, 0, PT_IESPECIAL2 /* Win98 */ };
 static const BYTE folderOrSpecialType[] = { PT_FOLDER, PT_IESPECIAL2 };
@@ -672,7 +675,7 @@ static void testWinDir(void)
  */
 static void testSystemDir(void)
 {
-    char systemShellPath[MAX_PATH], systemDir[MAX_PATH] = { 0 };
+    char systemShellPath[MAX_PATH], systemDir[MAX_PATH], systemDirx86[MAX_PATH];
 
     if (!pSHGetSpecialFolderPathA) return;
 
@@ -686,13 +689,13 @@ static void testSystemDir(void)
          systemDir, systemShellPath);
     }
 
-    if (!pGetSystemWow64DirectoryA || !pGetSystemWow64DirectoryA(systemDir, sizeof(systemDir)))
-        GetSystemDirectoryA(systemDir, sizeof(systemDir));
-    myPathRemoveBackslashA(systemDir);
+    if (!pGetSystemWow64DirectoryA || !pGetSystemWow64DirectoryA(systemDirx86, sizeof(systemDirx86)))
+        GetSystemDirectoryA(systemDirx86, sizeof(systemDirx86));
+    myPathRemoveBackslashA(systemDirx86);
     if (pSHGetSpecialFolderPathA(NULL, systemShellPath, CSIDL_SYSTEMX86, FALSE))
     {
         myPathRemoveBackslashA(systemShellPath);
-        ok(!lstrcmpiA(systemDir, systemShellPath),
+        ok(!lstrcmpiA(systemDirx86, systemShellPath) || broken(!lstrcmpiA(systemDir, systemShellPath)),
          "GetSystemDirectory returns %s SHGetSpecialFolderPath returns %s\n",
          systemDir, systemShellPath);
     }
