@@ -348,6 +348,7 @@ static void init_session(BOOL init)
 HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppv)
 {
     unsigned int i;
+    HRESULT hr;
     
     TRACE("(%s,%s,%p)\n", debugstr_guid(rclsid), debugstr_guid(riid), ppv);
     
@@ -357,6 +358,10 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppv)
 	    return IClassFactory_QueryInterface(object_creation[i].cf, riid, ppv);
     }
 
+    hr = URLMON_DllGetClassObject(rclsid, riid, ppv);
+    if(SUCCEEDED(hr))
+        return hr;
+
     FIXME("%s: no class found.\n", debugstr_guid(rclsid));
     return CLASS_E_CLASSNOTAVAILABLE;
 }
