@@ -33,7 +33,6 @@
 #include "user_private.h"
 #include "controls.h"
 #include "wine/unicode.h"
-#include "wine/winuser16.h"
 #include "wine/server.h"
 #include "wine/exception.h"
 #include "wine/debug.h"
@@ -44,6 +43,8 @@ WINE_DEFAULT_DEBUG_CHANNEL(win);
 #define KEYDATA_ALT             0x2000
 #define KEYDATA_PREVSTATE       0x4000
 
+#define DRAG_FILE  0x454C4946
+
 static short iF10Key = 0;
 static short iMenuSysKey = 0;
 static const WCHAR imm32W[] = { 'i','m','m','3','2','\0' };
