@@ -506,24 +506,6 @@ static void WINPOS_GetWinOffset( HWND hwndFrom, HWND hwndTo, POINT *offset )
 }
 
 
-/*******************************************************************
- *		MapWindowPoints (USER.258)
- */
-void WINAPI MapWindowPoints16( HWND16 hwndFrom, HWND16 hwndTo,
-                               LPPOINT16 lppt, UINT16 count )
-{
-    POINT offset;
-
-    WINPOS_GetWinOffset( WIN_Handle32(hwndFrom), WIN_Handle32(hwndTo), &offset );
-    while (count--)
-    {
-	lppt->x += offset.x;
-	lppt->y += offset.y;
-        lppt++;
-    }
-}
-
-
 /*******************************************************************
  *		MapWindowPoints (USER32.@)
  */
@@ -1056,7 +1038,7 @@ static BOOL show_window( HWND hwnd, INT cmd )
 
         /* Revert focus to parent */
         hFocus = GetFocus();
-        if (hwnd == hFocus || IsChild(hwnd, hFocus))
+        if (hwnd == hFocus)
         {
             HWND parent = GetAncestor(hwnd, GA_PARENT);
             if (parent == GetDesktopWindow()) parent = 0;
