@@ -221,6 +221,30 @@ static void test_OpenRequest (void)
 
 }
 
+static void test_empty_headers_param(void)
+{
+    static const WCHAR winehq[] = {'w','i','n','e','h','q','.','o','r','g',0};
+    static const WCHAR empty[]  = {0};
+    HANDLE ses, con, req;
+    BOOL ret;
+
+    ses = WinHttpOpen(test_useragent, 0, NULL, NULL, 0);
+    ok(ses != NULL, "failed to open session %u\n", GetLastError());
+
+    con = WinHttpConnect(ses, winehq, 80, 0);
+    ok(con != NULL, "failed to open a connection %u\n", GetLastError());
+
+    req = WinHttpOpenRequest(con, NULL, NULL, NULL, NULL, NULL, 0);
+    ok(req != NULL, "failed to open a request %u\n", GetLastError());
+
+    ret = WinHttpSendRequest(req, empty, 0, NULL, 0, 0, 0);
+    ok(ret, "failed to send request %u\n", GetLastError());
+
+    WinHttpCloseHandle(req);
+    WinHttpCloseHandle(con);
+    WinHttpCloseHandle(ses);
+}
+
 static void test_SendRequest (void)
 {
     HINTERNET session, request, connection;
@@ -759,11 +783,17 @@ static void test_secure_connection(void)
 
     ret = WinHttpSendRequest(req, NULL, 0, NULL, 0, 0, 0);
     ok(ret, "failed to send request %u\n", GetLastError());
+    if (!ret)
+    {
+        skip("secure connection failed, skipping remaining secure tests\n");
+        goto cleanup;
+    }
 
     size = sizeof(cert);
     ret = WinHttpQueryOption(req, WINHTTP_OPTION_SERVER_CERT_CONTEXT, &cert, &size );
     ok(ret, "failed to retrieve certificate context %u\n", GetLastError());
-    CertFreeCertificateContext(cert);
+    if (ret)
+        CertFreeCertificateContext(cert);
 
     size = sizeof(bitness);
     ret = WinHttpQueryOption(req, WINHTTP_OPTION_SECURITY_KEY_BITNESS, &bitness, &size );
@@ -781,6 +811,7 @@ static void test_secure_connection(void)
     ret = WinHttpQueryHeaders(req, WINHTTP_QUERY_RAW_HEADERS_CRLF, NULL, NULL, &size, NULL);
     ok(!ret, "succeeded unexpectedly\n");
 
+cleanup:
     WinHttpCloseHandle(req);
     WinHttpCloseHandle(con);
     WinHttpCloseHandle(ses);
@@ -792,7 +823,8 @@ static void test_request_parameter_defaults(void)
     static const WCHAR codeweavers[] = {'c','o','d','e','w','e','a','v','e','r','s','.','c','o','m',0};
 
     HANDLE ses, con, req;
-    DWORD size, status;
+    DWORD size, status, error;
+    WCHAR *version;
     BOOL ret;
 
     ses = WinHttpOpen(test_useragent, 0, NULL, NULL, 0);
@@ -826,6 +858,19 @@ static void test_request_parameter_defaults(void)
     ret = WinHttpReceiveResponse(req, NULL);
     ok(ret, "failed to receive response %u\n", GetLastError());
 
+    size = 0;
+    SetLastError(0xdeadbeef);
+    ret = WinHttpQueryHeaders(req, WINHTTP_QUERY_VERSION, NULL, NULL, &size, NULL);
+    error = GetLastError();
+    ok(!ret, "succeeded unexpectedly\n");
+    ok(error == ERROR_INSUFFICIENT_BUFFER, "expected ERROR_INSUFFICIENT_BUFFER, got %u\n", error);
+
+    version = HeapAlloc(GetProcessHeap(), 0, size);
+    ret = WinHttpQueryHeaders(req, WINHTTP_QUERY_VERSION, NULL, version, &size, NULL);
+    ok(ret, "failed unexpectedly %u\n", GetLastError());
+    ok(lstrlenW(version) == size / sizeof(WCHAR), "unexpected size %u\n", size);
+    HeapFree(GetProcessHeap(), 0, version);
+
     size = sizeof(status);
     ret = WinHttpQueryHeaders(req, WINHTTP_QUERY_STATUS_CODE | WINHTTP_QUERY_FLAG_NUMBER, NULL, &status, &size, NULL);
     ok(ret, "failed unexpectedly %u\n", GetLastError());
@@ -962,9 +1007,16 @@ static void test_set_default_proxy_config(void)
     info.lpszProxy = normalString;
     SetLastError(0xdeadbeef);
     ret = WinHttpSetDefaultProxyConfiguration(&info);
-    ok(ret, "WinHttpSetDefaultProxyConfiguration failed: %d\n", GetLastError());
-
-    set_default_proxy_reg_value( saved_proxy_settings, len, type );
+    if (ret)
+    {
+        ok(ret, "always true\n");
+        set_default_proxy_reg_value( saved_proxy_settings, len, type );
+    }
+    else if (GetLastError() == ERROR_ACCESS_DENIED)
+        skip("couldn't set default proxy configuration: access denied\n");
+    else
+        ok(ret, "WinHttpSetDefaultProxyConfiguration failed: %d\n",
+           GetLastError());
 }
 
 START_TEST (winhttp)
@@ -978,4 +1030,5 @@ START_TEST (winhttp)
     test_request_parameter_defaults();
     test_QueryOption();
     test_set_default_proxy_config();
+    test_empty_headers_param();
 }
