@@ -23,6 +23,8 @@
 #include "config.h"
 #include "wine/port.h"
 
+#define NONAMELESSUNION
+
 #if defined(__MINGW32__) || defined (_MSC_VER)
 #include <ws2tcpip.h>
 #endif
@@ -113,6 +115,8 @@ static void *OpenSSL_crypto_handle;
 
 static SSL_METHOD *meth;
 static SSL_CTX *ctx;
+static int hostname_idx;
+static int error_idx;
 
 #define MAKE_FUNCPTR(f) static typeof(f) * p##f
 
@@ -130,20 +134,29 @@ MAKE_FUNCPTR(SSL_shutdown);
 MAKE_FUNCPTR(SSL_write);
 MAKE_FUNCPTR(SSL_read);
 MAKE_FUNCPTR(SSL_pending);
+MAKE_FUNCPTR(SSL_get_ex_new_index);
+MAKE_FUNCPTR(SSL_get_ex_data);
+MAKE_FUNCPTR(SSL_set_ex_data);
+MAKE_FUNCPTR(SSL_get_ex_data_X509_STORE_CTX_idx);
 MAKE_FUNCPTR(SSL_get_verify_result);
 MAKE_FUNCPTR(SSL_get_peer_certificate);
 MAKE_FUNCPTR(SSL_CTX_get_timeout);
 MAKE_FUNCPTR(SSL_CTX_set_timeout);
 MAKE_FUNCPTR(SSL_CTX_set_default_verify_paths);
+MAKE_FUNCPTR(SSL_CTX_set_verify);
+MAKE_FUNCPTR(X509_STORE_CTX_get_ex_data);
 
 /* OpenSSL's libcrypto functions that we use */
 MAKE_FUNCPTR(BIO_new_fp);
 MAKE_FUNCPTR(CRYPTO_num_locks);
 MAKE_FUNCPTR(CRYPTO_set_id_callback);
 MAKE_FUNCPTR(CRYPTO_set_locking_callback);
+MAKE_FUNCPTR(ERR_free_strings);
 MAKE_FUNCPTR(ERR_get_error);
 MAKE_FUNCPTR(ERR_error_string);
 MAKE_FUNCPTR(i2d_X509);
+MAKE_FUNCPTR(sk_num);
+MAKE_FUNCPTR(sk_value);
 #undef MAKE_FUNCPTR
 
 static CRITICAL_SECTION *ssl_locks;
@@ -161,6 +174,165 @@ static void ssl_lock_callback(int mode, int type, const char *file, int line)
         LeaveCriticalSection(&ssl_locks[type]);
 }
 
+static PCCERT_CONTEXT X509_to_cert_context(X509 *cert)
+{
+    unsigned char* buffer,*p;
+    INT len;
+    BOOL malloced = FALSE;
+    PCCERT_CONTEXT ret;
+
+    p = NULL;
+    len = pi2d_X509(cert,&p);
+    /*
+     * SSL 0.9.7 and above malloc the buffer if it is null.
+     * however earlier version do not and so we would need to alloc the buffer.
+     *
+     * see the i2d_X509 man page for more details.
+     */
+    if (!p)
+    {
+        buffer = HeapAlloc(GetProcessHeap(),0,len);
+        p = buffer;
+        len = pi2d_X509(cert,&p);
+    }
+    else
+    {
+        buffer = p;
+        malloced = TRUE;
+    }
+
+    ret = CertCreateCertificateContext(X509_ASN_ENCODING,buffer,len);
+
+    if (malloced)
+        free(buffer);
+    else
+        HeapFree(GetProcessHeap(),0,buffer);
+
+    return ret;
+}
+
+static DWORD netconn_verify_cert(PCCERT_CONTEXT cert, HCERTSTORE store,
+    WCHAR *server)
+{
+    BOOL ret;
+    CERT_CHAIN_PARA chainPara = { sizeof(chainPara), { 0 } };
+    PCCERT_CHAIN_CONTEXT chain;
+    char oid_server_auth[] = szOID_PKIX_KP_SERVER_AUTH;
+    char *server_auth[] = { oid_server_auth };
+    DWORD err = ERROR_SUCCESS;
+
+    TRACE("verifying %s\n", debugstr_w(server));
+    chainPara.RequestedUsage.Usage.cUsageIdentifier = 1;
+    chainPara.RequestedUsage.Usage.rgpszUsageIdentifier = server_auth;
+    if ((ret = CertGetCertificateChain(NULL, cert, NULL, store, &chainPara, 0,
+        NULL, &chain)))
+    {
+        if (chain->TrustStatus.dwErrorStatus)
+        {
+            if (chain->TrustStatus.dwErrorStatus & CERT_TRUST_IS_NOT_TIME_VALID)
+                err = ERROR_INTERNET_SEC_CERT_DATE_INVALID;
+            else if (chain->TrustStatus.dwErrorStatus &
+                     CERT_TRUST_IS_UNTRUSTED_ROOT)
+                err = ERROR_INTERNET_INVALID_CA;
+            else if ((chain->TrustStatus.dwErrorStatus &
+                      CERT_TRUST_IS_OFFLINE_REVOCATION) ||
+                     (chain->TrustStatus.dwErrorStatus &
+                      CERT_TRUST_REVOCATION_STATUS_UNKNOWN))
+                err = ERROR_INTERNET_SEC_CERT_NO_REV;
+            else if (chain->TrustStatus.dwErrorStatus & CERT_TRUST_IS_REVOKED)
+                err = ERROR_INTERNET_SEC_CERT_REVOKED;
+            else if (chain->TrustStatus.dwErrorStatus &
+                CERT_TRUST_IS_NOT_VALID_FOR_USAGE)
+                err = ERROR_INTERNET_SEC_INVALID_CERT;
+            else
+                err = ERROR_INTERNET_SEC_INVALID_CERT;
+        }
+        else
+        {
+            CERT_CHAIN_POLICY_PARA policyPara;
+            SSL_EXTRA_CERT_CHAIN_POLICY_PARA sslExtraPolicyPara;
+            CERT_CHAIN_POLICY_STATUS policyStatus;
+
+            sslExtraPolicyPara.u.cbSize = sizeof(sslExtraPolicyPara);
+            sslExtraPolicyPara.dwAuthType = AUTHTYPE_SERVER;
+            sslExtraPolicyPara.pwszServerName = server;
+            policyPara.cbSize = sizeof(policyPara);
+            policyPara.dwFlags = 0;
+            policyPara.pvExtraPolicyPara = &sslExtraPolicyPara;
+            ret = CertVerifyCertificateChainPolicy(CERT_CHAIN_POLICY_SSL,
+                chain, &policyPara, &policyStatus);
+            /* Any error in the policy status indicates that the
+             * policy couldn't be verified.
+             */
+            if (ret && policyStatus.dwError)
+            {
+                if (policyStatus.dwError == CERT_E_CN_NO_MATCH)
+                    err = ERROR_INTERNET_SEC_CERT_CN_INVALID;
+                else
+                    err = ERROR_INTERNET_SEC_INVALID_CERT;
+            }
+        }
+        CertFreeCertificateChain(chain);
+    }
+    TRACE("returning %08x\n", err);
+    return err;
+}
+
+static int netconn_secure_verify(int preverify_ok, X509_STORE_CTX *ctx)
+{
+    SSL *ssl;
+    WCHAR *server;
+    BOOL ret = FALSE;
+
+    ssl = pX509_STORE_CTX_get_ex_data(ctx,
+        pSSL_get_ex_data_X509_STORE_CTX_idx());
+    server = pSSL_get_ex_data(ssl, hostname_idx);
+    if (preverify_ok)
+    {
+        HCERTSTORE store = CertOpenStore(CERT_STORE_PROV_MEMORY, 0, 0,
+            CERT_STORE_CREATE_NEW_FLAG, NULL);
+
+        if (store)
+        {
+            X509 *cert;
+            int i;
+            PCCERT_CONTEXT endCert = NULL;
+
+            ret = TRUE;
+            for (i = 0; ret && i < psk_num((struct stack_st *)ctx->chain); i++)
+            {
+                PCCERT_CONTEXT context;
+
+                cert = (X509 *)psk_value((struct stack_st *)ctx->chain, i);
+                if ((context = X509_to_cert_context(cert)))
+                {
+                    if (i == 0)
+                        ret = CertAddCertificateContextToStore(store, context,
+                            CERT_STORE_ADD_ALWAYS, &endCert);
+                    else
+                        ret = CertAddCertificateContextToStore(store, context,
+                            CERT_STORE_ADD_ALWAYS, NULL);
+                    CertFreeCertificateContext(context);
+                }
+            }
+            if (!endCert) ret = FALSE;
+            if (ret)
+            {
+                DWORD_PTR err = netconn_verify_cert(endCert, store, server);
+
+                if (err)
+                {
+                    pSSL_set_ex_data(ssl, error_idx, (void *)err);
+                    ret = FALSE;
+                }
+            }
+            CertFreeCertificateContext(endCert);
+            CertCloseStore(store, 0);
+        }
+    }
+    return ret;
+}
+
 #endif
 
 DWORD NETCON_init(WININET_NETCONNECTION *connection, BOOL useSSL)
@@ -219,11 +391,17 @@ DWORD NETCON_init(WININET_NETCONNECTION *connection, BOOL useSSL)
 	DYNSSL(SSL_write);
 	DYNSSL(SSL_read);
 	DYNSSL(SSL_pending);
+	DYNSSL(SSL_get_ex_new_index);
+	DYNSSL(SSL_get_ex_data);
+	DYNSSL(SSL_set_ex_data);
+	DYNSSL(SSL_get_ex_data_X509_STORE_CTX_idx);
 	DYNSSL(SSL_get_verify_result);
 	DYNSSL(SSL_get_peer_certificate);
 	DYNSSL(SSL_CTX_get_timeout);
 	DYNSSL(SSL_CTX_set_timeout);
 	DYNSSL(SSL_CTX_set_default_verify_paths);
+	DYNSSL(SSL_CTX_set_verify);
+	DYNSSL(X509_STORE_CTX_get_ex_data);
 #undef DYNSSL
 
 #define DYNCRYPTO(x) \
@@ -238,9 +416,12 @@ DWORD NETCON_init(WININET_NETCONNECTION *connection, BOOL useSSL)
 	DYNCRYPTO(CRYPTO_num_locks);
 	DYNCRYPTO(CRYPTO_set_id_callback);
 	DYNCRYPTO(CRYPTO_set_locking_callback);
+	DYNCRYPTO(ERR_free_strings);
 	DYNCRYPTO(ERR_get_error);
 	DYNCRYPTO(ERR_error_string);
 	DYNCRYPTO(i2d_X509);
+	DYNCRYPTO(sk_num);
+	DYNCRYPTO(sk_value);
 #undef DYNCRYPTO
 
 	pSSL_library_init();
@@ -256,6 +437,25 @@ DWORD NETCON_init(WININET_NETCONNECTION *connection, BOOL useSSL)
             LeaveCriticalSection(&init_ssl_cs);
             return ERROR_OUTOFMEMORY;
         }
+        hostname_idx = pSSL_get_ex_new_index(0, (void *)"hostname index",
+                NULL, NULL, NULL);
+        if (hostname_idx == -1)
+        {
+            ERR("SSL_get_ex_new_index failed; %s\n",
+                pERR_error_string(pERR_get_error(), 0));
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_OUTOFMEMORY;
+        }
+        error_idx = pSSL_get_ex_new_index(0, (void *)"error index",
+                NULL, NULL, NULL);
+        if (error_idx == -1)
+        {
+            ERR("SSL_get_ex_new_index failed; %s\n",
+                pERR_error_string(pERR_get_error(), 0));
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_OUTOFMEMORY;
+        }
+        pSSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, netconn_secure_verify);
 
         pCRYPTO_set_id_callback(ssl_thread_id);
         ssl_locks = HeapAlloc(GetProcessHeap(), 0,
@@ -282,6 +482,7 @@ void NETCON_unload(void)
 #if defined(SONAME_LIBSSL) && defined(SONAME_LIBCRYPTO)
     if (OpenSSL_crypto_handle)
     {
+        pERR_free_strings();
         wine_dlclose(OpenSSL_crypto_handle, NULL, 0);
     }
     if (OpenSSL_ssl_handle)
@@ -421,25 +622,17 @@ DWORD NETCON_close(WININET_NETCONNECTION *connection)
         return sock_get_error(errno);
     return ERROR_SUCCESS;
 }
-#ifdef SONAME_LIBSSL
-static BOOL check_hostname(X509 *cert, char *hostname)
-{
-    /* FIXME: implement */
-    return TRUE;
-}
-#endif
+
 /******************************************************************************
  * NETCON_secure_connect
  * Initiates a secure connection over an existing plaintext connection.
  */
-DWORD NETCON_secure_connect(WININET_NETCONNECTION *connection, LPCWSTR hostname)
+DWORD NETCON_secure_connect(WININET_NETCONNECTION *connection, LPWSTR hostname)
 {
     DWORD res = ERROR_NOT_SUPPORTED;
 #ifdef SONAME_LIBSSL
     long verify_res;
     X509 *cert;
-    int len;
-    char *hostname_unix;
 
     /* can't connect if we are already connected */
     if (connection->useSSL)
@@ -467,11 +660,13 @@ DWORD NETCON_secure_connect(WININET_NETCONNECTION *connection, LPCWSTR hostname)
 
     if (pSSL_connect(connection->ssl_s) <= 0)
     {
-        ERR("SSL_connect failed: %s\n",
-            pERR_error_string(pERR_get_error(), 0));
-        res = ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
+        res = (DWORD_PTR)pSSL_get_ex_data(connection->ssl_s, error_idx);
+        if (!res)
+            res = ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
+        ERR("SSL_connect failed: %d\n", res);
         goto fail;
     }
+    pSSL_set_ex_data(connection->ssl_s, hostname_idx, hostname);
     cert = pSSL_get_peer_certificate(connection->ssl_s);
     if (!cert)
     {
@@ -488,23 +683,6 @@ DWORD NETCON_secure_connect(WININET_NETCONNECTION *connection, LPCWSTR hostname)
          * the moment */
     }
 
-    len = WideCharToMultiByte(CP_UNIXCP, 0, hostname, -1, NULL, 0, NULL, NULL);
-    hostname_unix = HeapAlloc(GetProcessHeap(), 0, len);
-    if (!hostname_unix)
-    {
-        res = ERROR_OUTOFMEMORY;
-        goto fail;
-    }
-    WideCharToMultiByte(CP_UNIXCP, 0, hostname, -1, hostname_unix, len, NULL, NULL);
-
-    if (!check_hostname(cert, hostname_unix))
-    {
-        HeapFree(GetProcessHeap(), 0, hostname_unix);
-        res = ERROR_INTERNET_SEC_CERT_CN_INVALID;
-        goto fail;
-    }
-
-    HeapFree(GetProcessHeap(), 0, hostname_unix);
     connection->useSSL = TRUE;
     return ERROR_SUCCESS;
 
@@ -636,42 +814,13 @@ LPCVOID NETCON_GetCert(WININET_NETCONNECTION *connection)
 {
 #ifdef SONAME_LIBSSL
     X509* cert;
-    unsigned char* buffer,*p;
-    INT len;
-    BOOL malloced = FALSE;
     LPCVOID r = NULL;
 
     if (!connection->useSSL)
         return NULL;
 
     cert = pSSL_get_peer_certificate(connection->ssl_s);
-    p = NULL;
-    len = pi2d_X509(cert,&p);
-    /*
-     * SSL 0.9.7 and above malloc the buffer if it is null. 
-     * however earlier version do not and so we would need to alloc the buffer.
-     *
-     * see the i2d_X509 man page for more details.
-     */
-    if (!p)
-    {
-        buffer = HeapAlloc(GetProcessHeap(),0,len);
-        p = buffer;
-        len = pi2d_X509(cert,&p);
-    }
-    else
-    {
-        buffer = p;
-        malloced = TRUE;
-    }
-
-    r = CertCreateCertificateContext(X509_ASN_ENCODING,buffer,len);
-
-    if (malloced)
-        free(buffer);
-    else
-        HeapFree(GetProcessHeap(),0,buffer);
-
+    r = X509_to_cert_context(cert);
     return r;
 #else
     return NULL;
