@@ -83,7 +83,7 @@ HINSTANCE hinst;
 DWORD errorlevel;
 int echo_mode = 1, verify_mode = 0, defaultColor = 7;
 static int opt_c, opt_k, opt_s;
-const WCHAR newline[] = {'\n','\0'};
+const WCHAR newline[] = {'\r','\n','\0'};
 static const WCHAR equalsW[] = {'=','\0'};
 static const WCHAR closeBW[] = {')','\0'};
 WCHAR anykey[100];
@@ -325,6 +325,8 @@ static void WCMD_show_prompt (void) {
   }
   p = prompt_string;
   q = out_string;
+  *q++ = '\r';
+  *q++ = '\n';
   *q = '\0';
   while (*p != '\0') {
     if (*p != '$') {
@@ -1267,13 +1269,6 @@ void WCMD_execute (WCHAR *command, WCHAR *redirects,
     handleExpansion(new_redir, (context != NULL), forVariable, forValue);
     cmd = new_cmd;
 
-    /* Show prompt before batch line IF echo is on and in batch program */
-    if (context && echo_mode && (cmd[0] != '@')) {
-      WCMD_show_prompt();
-      WCMD_output_asis ( cmd);
-      WCMD_output_asis ( newline);
-    }
-
 /*
  *	Changing default drive has to be handled as a special case.
  */
@@ -1772,6 +1767,12 @@ WCHAR *WCMD_ReadAndParseLine(WCHAR *optionalcmd, CMD_LIST **output, HANDLE readF
 
     /* Replace env vars if in a batch context */
     if (context) handleExpansion(extraSpace, FALSE, NULL, NULL);
+    /* Show prompt before batch line IF echo is on and in batch program */
+    if (context && echo_mode && extraSpace[0] && (extraSpace[0] != '@')) {
+      WCMD_show_prompt();
+      WCMD_output_asis(extraSpace);
+      WCMD_output_asis(newline);
+    }
 
     /* Start with an empty string, copying to the command string */
     curStringLen = 0;
@@ -2104,6 +2105,11 @@ WCHAR *WCMD_ReadAndParseLine(WCHAR *optionalcmd, CMD_LIST **output, HANDLE readF
         }
         curPos = extraSpace;
         if (context) handleExpansion(extraSpace, FALSE, NULL, NULL);
+        /* Continue to echo commands IF echo is on and in batch program */
+        if (context && echo_mode && extraSpace[0] && (extraSpace[0] != '@')) {
+          WCMD_output_asis(extraSpace);
+          WCMD_output_asis(newline);
+        }
       }
     }
 
@@ -2189,6 +2195,8 @@ int wmain (int argc, WCHAR *argvW[])
   int opt_t = 0;
   static const WCHAR autoexec[] = {'\\','a','u','t','o','e','x','e','c','.',
                                    'b','a','t','\0'};
+  static const WCHAR promptW[] = {'P','R','O','M','P','T','\0'};
+  static const WCHAR defaultpromptW[] = {'$','P','$','G','\0'};
   char ansiVersion[100];
   CMD_LIST *toExecute = NULL;         /* Commands left to be executed */
 
@@ -2521,6 +2529,7 @@ int wmain (int argc, WCHAR *argvW[])
  *	Loop forever getting commands and executing them.
  */
 
+  SetEnvironmentVariableW(promptW, defaultpromptW);
   WCMD_version ();
   while (TRUE) {
 
